@@ -72,7 +72,7 @@ class Extension2ReaderTable(QWidget):
                 remove_btn.setFixedWidth(30)
                 remove_btn.setStyleSheet('margin: 4px;')
                 remove_btn.setToolTip(
-                    'Remove this extension to reader association'
+                    trans._('Remove this extension to reader association')
                 )
                 remove_btn.clicked.connect(self._remove_extension_assignment)
 
