@@ -18,7 +18,6 @@ To generate a screenshot within python:
 
 from qtpy.QtCore import Qt
 from qtpy.QtWidgets import (
-    QApplication,
     QCheckBox,
     QComboBox,
     QDoubleSpinBox,
@@ -40,14 +39,11 @@ from qtpy.QtWidgets import (
     QVBoxLayout,
     QWidget,
 )
+from superqt import QRangeSlider
 
-from ...resources import get_stylesheet
 from ...utils.io import imsave
-from ...utils.theme import palettes, template
+from ..qt_resources import get_stylesheet
 from ..utils import QImg2array
-from .qt_range_slider import QHRangeSlider
-
-raw_stylesheet = get_stylesheet()
 
 blurb = """
 <h3>Heading</h3>
@@ -95,7 +91,7 @@ class SampleWidget(QWidget):
     def __init__(self, theme='dark', emphasized=False):
         super().__init__(None)
         self.setProperty('emphasized', emphasized)
-        self.setStyleSheet(template(raw_stylesheet, **palettes[theme]))
+        self.setStyleSheet(get_stylesheet(theme))
         lay = QVBoxLayout()
         self.setLayout(lay)
         lay.addWidget(QPushButton('push button'))
@@ -124,7 +120,7 @@ class SampleWidget(QWidget):
         scroll = QScrollBar(Qt.Horizontal)
         scroll.setValue(50)
         lay.addWidget(scroll)
-        lay.addWidget(QHRangeSlider(parent=self))
+        lay.addWidget(QRangeSlider(Qt.Horizontal, self))
         text = QTextEdit()
         text.setMaximumHeight(100)
         text.setHtml(blurb)
@@ -160,8 +156,11 @@ class SampleWidget(QWidget):
 if __name__ == "__main__":
     import sys
 
-    themes = [sys.argv[1]] if len(sys.argv) > 1 else palettes.keys()
-    app = QApplication([])
+    from ...utils.theme import available_themes
+    from ..qt_event_loop import get_app
+
+    themes = [sys.argv[1]] if len(sys.argv) > 1 else available_themes()
+    app = get_app()
     widgets = []
     for n, theme in enumerate(themes):
         try:
