@@ -29,11 +29,12 @@ from ..layers.utils.stack_utils import split_channels
 from ..settings import get_settings
 from ..utils._register import create_func as create_add_method
 from ..utils.colormaps import ensure_colormap
+from ..utils.context import Context, create_context
 from ..utils.events import Event, EventedModel, disconnect_events
-from ..utils.events.event import WarningEmitter
 from ..utils.key_bindings import KeymapProvider
 from ..utils.misc import is_sequence
 from ..utils.mouse_bindings import MousemapProvider
+from ..utils.progress import progress
 from ..utils.theme import available_themes
 from ..utils.translations import trans
 from ._viewer_mouse_bindings import dims_scroll
@@ -43,6 +44,7 @@ from .cursor import Cursor
 from .dims import Dims
 from .grid import GridCanvas
 from .layerlist import LayerList
+from .overlays import Overlays
 from .scale_bar import ScaleBar
 from .text_overlay import TextOverlay
 from .tooltip import Tooltip
@@ -114,6 +116,7 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
     text_overlay: TextOverlay = Field(
         default_factory=TextOverlay, allow_mutation=False
     )
+    overlays: Overlays = Field(default_factory=Overlays, allow_mutation=False)
 
     help: str = ''
     status: str = 'Ready'
@@ -123,8 +126,11 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
 
     # 2-tuple indicating height and width
     _canvas_size: Tuple[int, int] = (600, 800)
+    _ctx: Context
 
     def __init__(self, title='napari', ndisplay=2, order=(), axis_labels=()):
+        # max_depth=0 means don't look for parent contexts.
+        self._ctx = create_context(self, max_depth=0)
         # allow extra attributes during model initialization, useful for mixins
         self.__config__.extra = Extra.allow
         super().__init__(
@@ -175,21 +181,10 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         # Add mouse callback
         self.mouse_wheel_callbacks.append(dims_scroll)
 
-        self.events.add(
-            # FIXME: Deferred translation?
-            active_layer=WarningEmitter(
-                trans._(
-                    "'viewer.events.active_layer' is deprecated and will be removed in napari v0.4.9, use 'viewer.layers.selection.events.active' instead",
-                    deferred=True,
-                ),
-                type='active_layer',
-            )
-        )
-
     def _tooltip_visible_update(self, event):
         self.tooltip.visible = event.value
 
-    def _update_viewer_grid(self, e=None):
+    def _update_viewer_grid(self):
         """Keep viewer grid settings up to date with settings values."""
 
         settings = get_settings()
@@ -262,7 +257,7 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         else:
             return self.layers.extent.world[:, self.dims.displayed]
 
-    def reset_view(self, event=None):
+    def reset_view(self):
         """Reset the camera view."""
 
         extent = self._sliced_extent_world
@@ -301,15 +296,15 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         extent = self.layers.extent.world
         scale = self.layers.extent.step
         scene_size = extent[1] - extent[0]
-        corner = extent[0]
+        corner = extent[0] + 0.5 * self.layers.extent.step
         shape = [
-            np.round(s / sc).astype('int') + 1 if s > 0 else 1
+            np.round(s / sc).astype('int') if s > 0 else 1
             for s, sc in zip(scene_size, scale)
         ]
         empty_labels = np.zeros(shape, dtype=int)
         self.add_labels(empty_labels, translate=np.array(corner), scale=scale)
 
-    def _update_layers(self, event=None, layers=None):
+    def _update_layers(self, *, layers=None):
         """Updates the contained layers.
 
         Parameters
@@ -336,46 +331,19 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
             self.cursor.size = active_layer.cursor_size
             self.camera.interactive = active_layer.interactive
 
-    @property
-    def active_layer(self):
-        warnings.warn(
-            trans._(
-                "'viewer.active_layer' is deprecated and will be removed in napari v0.4.9.  Please use 'viewer.layers.selection.active' instead.",
-                deferred=True,
-            ),
-            category=FutureWarning,
-            stacklevel=2,
-        )
-        return self.layers.selection.active
+    @staticmethod
+    def rounded_division(min_val, max_val, precision):
+        return int(((min_val + max_val) / 2) / precision) * precision
 
-    def __setattr__(self, name: str, value: Any) -> None:
-        # this method is only for the deprecation warning, because pydantic
-        # prevents using @active_layer.setter
-        if name != 'active_layer':
-            return super().__setattr__(name, value)
-
-        warnings.warn(
-            trans._(
-                "'viewer.active_layer' is deprecated and will be removed in napari v0.4.9.  Please use 'viewer.layers.selection.active' instead.",
-                deferred=True,
-            ),
-            category=FutureWarning,
-            stacklevel=2,
-        )
-        self.layers.selection.active = value
-
-    def _on_layers_change(self, event):
+    def _on_layers_change(self):
         if len(self.layers) == 0:
             self.dims.ndim = 2
             self.dims.reset()
         else:
-            extent = self.layers.extent
-            world = extent.world
-            ss = extent.step
-            ndim = world.shape[1]
+            ranges = self.layers._ranges
+            ndim = len(ranges)
             self.dims.ndim = ndim
-            for i in range(ndim):
-                self.dims.set_range(i, (world[0, i], world[1, i], ss[i]))
+            self.dims.set_range(range(ndim), ranges)
 
         new_dim = self.dims.ndim
         dim_diff = new_dim - len(self.cursor.position)
@@ -399,7 +367,7 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         """Set the viewer cursor_size with the `event.cursor_size` int."""
         self.cursor.size = event.cursor_size
 
-    def _on_cursor_position_change(self, event):
+    def _on_cursor_position_change(self):
         """Set the layer cursor position."""
         with warnings.catch_warnings():
             # Catch the deprecation warning on layer.position
@@ -424,10 +392,13 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
             self.help = active.help
             if self.tooltip.visible:
                 self.tooltip.text = active._get_tooltip_text(
-                    self.cursor.position, world=True
+                    self.cursor.position,
+                    view_direction=self.cursor._view_direction,
+                    dims_displayed=list(self.dims.displayed),
+                    world=True,
                 )
 
-    def _on_grid_change(self, event):
+    def _on_grid_change(self):
         """Arrange the current layers is a 2D grid."""
         extent = self._sliced_extent_world
         n_layers = len(self.layers)
@@ -447,11 +418,11 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         extent : array, shape (2, D)
             Extent of the world.
         """
-        scene_shift = extent[1] - extent[0] + 1
+        scene_shift = extent[1] - extent[0]
         translate_2d = np.multiply(scene_shift[-2:], position)
         translate = [0] * layer.ndim
         translate[-2:] = translate_2d
-        layer.translate_grid = translate
+        layer._translate_grid = translate
 
     @property
     def experimental(self):
@@ -488,13 +459,16 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         layer.events.name.connect(self.layers._update_name)
 
         # Update dims and grid model
-        self._on_layers_change(None)
-        self._on_grid_change(None)
+        self._on_layers_change()
+        self._on_grid_change()
         # Slice current layer based on dims
         self._update_layers(layers=[layer])
 
         if len(self.layers) == 1:
             self.reset_view()
+            ranges = self.layers._ranges
+            midpoint = [self.rounded_division(*_range) for _range in ranges]
+            self.dims.set_point(range(len(ranges)), midpoint)
 
     def _on_remove_layer(self, event):
         """Disconnect old layer events.
@@ -515,8 +489,8 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         disconnect_events(layer.events, self)
         disconnect_events(layer.events, self.layers)
 
-        self._on_layers_change(None)
-        self._on_grid_change(None)
+        self._on_layers_change()
+        self._on_grid_change()
 
     def add_layer(self, layer: Layer) -> Layer:
         """Add a layer to the viewer.
@@ -562,6 +536,7 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         blending=None,
         visible=True,
         multiscale=None,
+        cache=True,
         experimental_slicing_plane=None,
         experimental_clipping_planes=None,
     ) -> Union[Image, List[Image]]:
@@ -673,6 +648,9 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
             should be the largest. Please note multiscale rendering is only
             supported in 2D. In 3D, only the lowest resolution scale is
             displayed.
+        cache : bool
+            Whether slices of out-of-core datasets should be cached upon
+            retrieval. Currently, this only applies to dask arrays.
         experimental_slicing_plane : dict or SlicingPlane
             Properties defining plane rendering in 3D. Properties are defined in
             data coordinates. Valid dictionary keys are
@@ -719,6 +697,7 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
             'blending': blending,
             'visible': visible,
             'multiscale': multiscale,
+            'cache': cache,
             'experimental_slicing_plane': experimental_slicing_plane,
             'experimental_clipping_planes': experimental_clipping_planes,
         }
@@ -785,9 +764,9 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         reader_plugin : str, optional
             reader plugin to pass to viewer.open (only used if the sample data
             is a string).  by default None.
-        **kwargs
+        ``**kwargs``
             additional kwargs will be passed to the sample data loader provided
-            by `plugin`.  Use of **kwargs may raise an error if the kwargs do
+            by `plugin`.  Use of ``**kwargs`` may raise an error if the kwargs do
             not match the sample data loader.
 
         Returns
@@ -800,25 +779,32 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
         KeyError
             If `plugin` does not provide a sample named `sample`.
         """
-        from ..plugins import plugin_manager
+        from ..plugins import _npe2, plugin_manager
 
-        try:
-            data = plugin_manager._sample_data[plugin][sample]['data']
-        except KeyError:
-            samples = plugin_manager.available_samples()
+        # try with npe2
+        data, available = _npe2.get_sample_data(plugin, sample)
+
+        # then try with npe1
+        if data is None:
+            try:
+                data = plugin_manager._sample_data[plugin][sample]['data']
+            except KeyError:
+                available += list(plugin_manager.available_samples())
+
+        if data is None:
             msg = trans._(
                 "Plugin {plugin!r} does not provide sample data named {sample!r}. ",
                 plugin=plugin,
                 sample=sample,
                 deferred=True,
             )
-            if samples:
+            if available:
                 msg = trans._(
                     "Plugin {plugin!r} does not provide sample data named {sample!r}. Available samples include: {samples}.",
                     deferred=True,
                     plugin=plugin,
                     sample=sample,
-                    samples=samples,
+                    samples=available,
                 )
             else:
                 msg = trans._(
@@ -884,7 +870,7 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
             additional) ``kwargs`` provided to this function.  This *may*
             result in exceptions if the data returned from the path is not
             compatible with the layer_type.
-        **kwargs
+        ``**kwargs``
             All other keyword arguments will be passed on to the respective
             ``add_layer`` method.
 
@@ -909,12 +895,19 @@ class ViewerModel(KeymapProvider, MousemapProvider, EventedModel):
             )
 
         added: List[Layer] = []  # for layers that get added
-        for _path in paths:
-            added.extend(
-                self._add_layers_with_plugins(
-                    _path, kwargs, plugin=plugin, layer_type=layer_type
+        with progress(
+            paths,
+            desc=trans._('Opening Files'),
+            total=0
+            if len(paths) == 1
+            else None,  # indeterminate bar for 1 file
+        ) as pbr:
+            for _path in pbr:
+                added.extend(
+                    self._add_layers_with_plugins(
+                        _path, kwargs, plugin=plugin, layer_type=layer_type
+                    )
                 )
-            )
         return added
 
     def _add_layers_with_plugins(
@@ -1260,5 +1253,5 @@ for _layer in (
     layers.Tracks,
     layers.Vectors,
 ):
-    func = create_add_method(_layer)
+    func = create_add_method(_layer, filename=__file__)
     setattr(ViewerModel, func.__name__, func)
