@@ -1,8 +1,15 @@
 import numpy as np
+
+from ...utils.geometry import (
+    inside_triangles,
+    intersect_line_with_triangles,
+    line_in_triangles_3d,
+)
+from ...utils.translations import trans
 from ._mesh import Mesh
-from ._shapes_models import Shape, Line, Path
-from ._shapes_utils import inside_triangles, triangles_intersect_box
-from ._shapes_constants import shape_classes, ShapeType
+from ._shapes_constants import ShapeType, shape_classes
+from ._shapes_models import Line, Path, Shape
+from ._shapes_utils import triangles_intersect_box
 
 
 class ShapeList:
@@ -40,8 +47,8 @@ class ShapeList:
     z_indices : (N, ) list of int
         z-index for each shape.
 
-    Extended Summary
-    ----------
+    Notes
+    -----
     _vertices : np.ndarray
         Mx2 array of all displayed vertices from all shapes
     _index : np.ndarray
@@ -133,7 +140,7 @@ class ShapeList:
         self._set_color(face_color, 'face')
 
     def _set_color(self, colors, attribute):
-        """ Set the face_color or edge_color property
+        """Set the face_color or edge_color property
 
         Parameters
         ----------
@@ -147,7 +154,12 @@ class ShapeList:
         n_shapes = len(self.data)
         if not np.all(colors.shape == (n_shapes, 4)):
             raise ValueError(
-                f'{attribute}_color must have shape ({n_shapes}, 4)'
+                trans._(
+                    '{attribute}_color must have shape ({n_shapes}, 4)',
+                    deferred=True,
+                    attribute=attribute,
+                    n_shapes=n_shapes,
+                )
             )
 
         update_method = getattr(self, f'update_{attribute}_color')
@@ -210,7 +222,14 @@ class ShapeList:
         self.displayed_vertices = self._vertices[disp_vert]
         self.displayed_index = self._index[disp_vert]
 
-    def add(self, shape, face_color=None, edge_color=None, shape_index=None):
+    def add(
+        self,
+        shape,
+        face_color=None,
+        edge_color=None,
+        shape_index=None,
+        z_refresh=True,
+    ):
         """Adds a single Shape object
 
         Parameters
@@ -222,12 +241,22 @@ class ShapeList:
             If int then edits the shape date at current index. To be used in
             conjunction with `remove` when renumber is `False`. If None, then
             appends a new shape to end of shapes list
+        z_refresh : bool
+            If set to true, the mesh elements are reindexed with the new z order.
+            When shape_index is provided, z_refresh will be overwritten to false,
+            as the z indices will not change.
+            When adding a batch of shapes, set to false  and then call
+            ShapesList._update_z_order() once at the end.
         """
         if not issubclass(type(shape), Shape):
-            raise ValueError('shape must be subclass of Shape')
+            raise ValueError(
+                trans._(
+                    'shape must be subclass of Shape',
+                    deferred=True,
+                )
+            )
 
         if shape_index is None:
-            z_refresh = True
             shape_index = len(self.shapes)
             self.shapes.append(shape)
             self._z_index = np.append(self._z_index, shape.z_index)
@@ -325,8 +354,7 @@ class ShapeList:
             self._update_z_order()
 
     def remove_all(self):
-        """Removes all shapes
-        """
+        """Removes all shapes"""
         self.shapes = []
         self._vertices = np.empty((0, self.ndisplay))
         self._index = np.empty((0), dtype=int)
@@ -420,8 +448,7 @@ class ShapeList:
             self._update_displayed()
 
     def _update_z_order(self):
-        """Updates the z order of the triangles given the z_index list
-        """
+        """Updates the z order of the triangles given the z_index list"""
         self._z_order = np.argsort(self._z_index)
         if len(self._z_order) == 0:
             self._mesh.triangles_z_order = np.empty((0), dtype=int)
@@ -461,7 +488,12 @@ class ShapeList:
                     shape_cls = shape_classes[shape_type]
                 else:
                     raise ValueError(
-                        f'{shape_type} must be one of {set(shape_classes)}'
+                        trans._(
+                            '{shape_type} must be one of {shape_classes}',
+                            deferred=True,
+                            shape_type=shape_type,
+                            shape_classes=set(shape_classes),
+                        )
                     )
             else:
                 shape_cls = new_type
@@ -757,6 +789,86 @@ class ShapeList:
         else:
             return None
 
+    def _inside_3d(self, ray_position: np.ndarray, ray_direction: np.ndarray):
+        """Determines if any shape is intersected by a ray by looking inside triangle
+        meshes. Looks only at displayed shapes.
+
+        Parameters
+        ----------
+        ray_position : np.ndarray
+            (3,) array containing the location that was clicked. This
+            should be in the same coordinate system as the vertices.
+        ray_direction : np.ndarray
+            (3,) array describing the direction camera is pointing in
+            the scene. This should be in the same coordinate system as
+            the vertices.
+
+        Returns
+        -------
+        shape : int | None
+            Index of shape if any that is at the coordinates. Returns `None`
+            if no shape is found.
+        intersection_point : Optional[np.ndarray]
+            The point where the ray intersects the mesh face. If there was
+            no intersection, returns None.
+        """
+        triangles = self._mesh.vertices[self._mesh.displayed_triangles]
+        inside = line_in_triangles_3d(
+            line_point=ray_position,
+            line_direction=ray_direction,
+            triangles=triangles,
+        )
+        intersected_shapes = self._mesh.displayed_triangles_index[inside, 0]
+        if len(intersected_shapes) > 0:
+            intersection_points = self._triangle_intersection(
+                triangle_indices=inside,
+                ray_position=ray_position,
+                ray_direction=ray_direction,
+            )
+            start_to_intersection = intersection_points - ray_position
+            distances = np.linalg.norm(start_to_intersection, axis=1)
+            closest_shape_index = np.argmin(distances)
+            shape = intersected_shapes[closest_shape_index]
+            intersection = intersection_points[closest_shape_index]
+            return shape, intersection
+        else:
+            return None, None
+
+    def _triangle_intersection(
+        self,
+        triangle_indices: np.ndarray,
+        ray_position: np.ndarray,
+        ray_direction: np.ndarray,
+    ):
+        """Find the intersection of a ray with specified triangles.
+
+        Parameters
+        ----------
+        triangle_indices : np.ndarray
+            (n,) array of shape indices to find the intersection with the ray. The indices should
+            correspond with self._mesh.displayed_triangles.
+        ray_position : np.ndarray
+            (3,) array with the coordinate of the starting point of the ray in layer coordinates.
+            Only provide the 3 displayed dimensions.
+        ray_direction : np.ndarray
+            (3,) array of the normal direction of the ray in layer coordinates.
+            Only provide the 3 displayed dimensions.
+
+        Returns
+        -------
+        intersection_points : np.ndarray
+            (n x 3) array of the intersection of the ray with each of the specified shapes in layer coordinates.
+            Only the 3 displayed dimensions are provided.
+        """
+        triangles = self._mesh.vertices[self._mesh.displayed_triangles]
+        intersected_triangles = triangles[triangle_indices]
+        intersection_points = intersect_line_with_triangles(
+            line_point=ray_position,
+            line_direction=ray_direction,
+            triangles=intersected_triangles,
+        )
+        return intersection_points
+
     def to_masks(self, mask_shape=None, zoom_factor=1, offset=[0, 0]):
         """Returns N binary masks, one for each shape, embedded in an array of
         shape `mask_shape`.
@@ -765,7 +877,7 @@ class ShapeList:
         ----------
         mask_shape : np.ndarray | tuple | None
             2-tuple defining shape of mask to be generated. If non specified,
-            takes the max of all the vertiecs
+            takes the max of all the vertices
         zoom_factor : float
             Premultiplier applied to coordinates before generating mask. Used
             for generating as downsampled mask.
@@ -801,7 +913,7 @@ class ShapeList:
         ----------
         labels_shape : np.ndarray | tuple | None
             2-tuple defining shape of labels image to be generated. If non
-            specified, takes the max of all the vertiecs
+            specified, takes the max of all the vertices
         zoom_factor : float
             Premultiplier applied to coordinates before generating mask. Used
             for generating as downsampled mask.
@@ -828,7 +940,9 @@ class ShapeList:
 
         return labels
 
-    def to_colors(self, colors_shape=None, zoom_factor=1, offset=[0, 0]):
+    def to_colors(
+        self, colors_shape=None, zoom_factor=1, offset=[0, 0], max_shapes=None
+    ):
         """Rasterize shapes to an RGBA image array.
 
         Each shape is embedded in an array of shape `colors_shape` with the
@@ -846,6 +960,11 @@ class ShapeList:
         offset : 2-tuple
             Offset subtracted from coordinates before multiplying by the
             zoom_factor. Used for putting negative coordinates into the mask.
+        max_shapes : None | int
+            If provided, this is the maximum number of shapes that will be rasterized.
+            If the number of shapes in view exceeds max_shapes, max_shapes shapes
+            will be randomly selected from the in view shapes. If set to None, no
+            maximum is applied. The default value is None.
 
         Returns
         -------
@@ -859,15 +978,24 @@ class ShapeList:
         colors = np.zeros(tuple(colors_shape) + (4,), dtype=float)
         colors[..., 3] = 1
 
-        for ind in self._z_order[::-1]:
-            if self._displayed[ind]:
-                mask = self.shapes[ind].to_mask(
-                    colors_shape, zoom_factor=zoom_factor, offset=offset
-                )
-                if type(self.shapes[ind]) in [Path, Line]:
-                    col = self._edge_color[ind]
-                else:
-                    col = self._face_color[ind]
-                colors[mask, :] = col
+        z_order = self._z_order[::-1]
+        shapes_in_view = np.argwhere(self._displayed)
+        z_order_in_view_mask = np.isin(z_order, shapes_in_view)
+        z_order_in_view = z_order[z_order_in_view_mask]
+
+        # If there are too many shapes to render responsively, just render
+        # the top max_shapes shapes
+        if max_shapes is not None and len(z_order_in_view) > max_shapes:
+            z_order_in_view = z_order_in_view[0:max_shapes]
+
+        for ind in z_order_in_view:
+            mask = self.shapes[ind].to_mask(
+                colors_shape, zoom_factor=zoom_factor, offset=offset
+            )
+            if type(self.shapes[ind]) in [Path, Line]:
+                col = self._edge_color[ind]
+            else:
+                col = self._face_color[ind]
+            colors[mask, :] = col
 
         return colors
