@@ -1,4 +1,5 @@
 import numpy as np
+import pytest
 
 from napari._tests.utils import check_layer_world_data_extent
 from napari.layers import Surface
@@ -21,6 +22,22 @@ def test_random_surface():
     assert layer._view_vertex_values.ndim == 1
 
 
+def test_random_surface_no_values():
+    """Test instantiating Surface layer with random 2D data but no vertex values."""
+    np.random.seed(0)
+    vertices = np.random.random((10, 2))
+    faces = np.random.randint(10, size=(6, 3))
+    data = (vertices, faces)
+    layer = Surface(data)
+    assert layer.ndim == 2
+    assert np.all([np.all(ld == d) for ld, d in zip(layer.data, data)])
+    assert np.all(layer.vertices == vertices)
+    assert np.all(layer.faces == faces)
+    assert np.all(layer.vertex_values == np.ones(len(vertices)))
+    assert layer._data_view.shape[1] == 2
+    assert layer._view_vertex_values.ndim == 1
+
+
 def test_random_3D_surface():
     """Test instantiating Surface layer with random 3D data."""
     np.random.seed(0)
@@ -69,7 +86,7 @@ def test_random_3D_timeseries_surface():
     assert np.all([np.all(ld == d) for ld, d in zip(layer.data, data)])
     assert layer._data_view.shape[1] == 2
     assert layer._view_vertex_values.ndim == 1
-    assert layer.shape[0] == 22
+    assert layer.extent.data[1][0] == 22
 
     layer._slice_dims(ndisplay=3)
     assert layer._data_view.shape[1] == 3
@@ -77,8 +94,9 @@ def test_random_3D_timeseries_surface():
 
     # If a values axis is made to be a displayed axis then no data should be
     # shown
-    layer._slice_dims(ndisplay=3, order=[3, 0, 1, 2])
-    assert len(layer._data_view) == 0
+    with pytest.warns(UserWarning):
+        layer._slice_dims(ndisplay=3, order=[3, 0, 1, 2])
+        assert len(layer._data_view) == 0
 
 
 def test_random_3D_multitimeseries_surface():
@@ -93,8 +111,32 @@ def test_random_3D_multitimeseries_surface():
     assert np.all([np.all(ld == d) for ld, d in zip(layer.data, data)])
     assert layer._data_view.shape[1] == 2
     assert layer._view_vertex_values.ndim == 1
-    assert layer.shape[0] == 16
-    assert layer.shape[1] == 22
+    assert layer.extent.data[1][0] == 16
+    assert layer.extent.data[1][1] == 22
+
+    layer._slice_dims(ndisplay=3)
+    assert layer._data_view.shape[1] == 3
+    assert layer._view_vertex_values.ndim == 1
+
+
+def test_changing_surface():
+    """Test changing surface layer data"""
+    np.random.seed(0)
+    vertices = np.random.random((10, 2))
+    faces = np.random.randint(10, size=(6, 3))
+    values = np.random.random(10)
+    data = (vertices, faces, values)
+    layer = Surface(data)
+
+    vertices = np.random.random((10, 3))
+    faces = np.random.randint(10, size=(6, 3))
+    values = np.random.random(10)
+    data = (vertices, faces, values)
+    layer.data = data
+    assert layer.ndim == 3
+    assert np.all([np.all(ld == d) for ld, d in zip(layer.data, data)])
+    assert layer._data_view.shape[1] == 2
+    assert layer._view_vertex_values.ndim == 1
 
     layer._slice_dims(ndisplay=3)
     assert layer._data_view.shape[1] == 3
@@ -148,4 +190,23 @@ def test_world_data_extent():
     max_val = (30, 15)
     layer = Surface((np.array(data), np.array((0, 1, 2)), np.array((0, 0, 0))))
     extent = np.array((min_val, max_val))
-    check_layer_world_data_extent(layer, extent, (3, 1), (20, 5))
+    check_layer_world_data_extent(layer, extent, (3, 1), (20, 5), False)
+
+
+def test_shading():
+    """Test setting shading"""
+    np.random.seed(0)
+    vertices = np.random.random((10, 3))
+    faces = np.random.randint(10, size=(6, 3))
+    values = np.random.random(10)
+    data = (vertices, faces, values)
+    layer = Surface(data)
+
+    # change shading property
+    shading = 'flat'
+    layer.shading = shading
+    assert layer.shading == shading
+
+    # set shading as keyword argument
+    layer = Surface(data, shading=shading)
+    assert layer.shading == shading
