@@ -1,5 +1,6 @@
 import importlib
 import sys
+import warnings
 from functools import partial
 from pathlib import Path
 from types import FunctionType
@@ -18,14 +19,16 @@ from warnings import warn
 
 from napari_plugin_engine import HookImplementation
 from napari_plugin_engine import PluginManager as PluginManager
+from napari_plugin_engine.hooks import HookCaller
+from pydantic import ValidationError
 from typing_extensions import TypedDict
 
+from ..settings import get_settings
 from ..types import AugmentedWidget, LayerData, SampleDict, WidgetCallable
-from ..utils import _magicgui
 from ..utils._appdirs import user_site_packages
 from ..utils.events import EmitterGroup, EventedSet
 from ..utils.misc import camel_to_spaces, running_as_bundled_app
-from ..utils.settings import SETTINGS
+from ..utils.theme import Theme, register_theme, unregister_theme
 from ..utils.translations import trans
 from . import _builtins, hook_specifications
 
@@ -43,15 +46,18 @@ CallOrderDict = Dict[str, List[PluginHookOption]]
 class NapariPluginManager(PluginManager):
     """PluginManager subclass for napari-specific functionality.
 
-    Events
-    ------
-    registered (value: str)
+    Notes
+    -----
+
+    The events emitted by the plugin include:
+
+    * registered (value: str)
         Emitted after plugin named `value` has been registered.
-    unregistered (value: str)
+    * unregistered (value: str)
         Emitted after plugin named `value` has been unregistered.
-    enabled (value: str)
+    * enabled (value: str)
         Emitted after plugin named `value` has been removed from the block list.
-    disabled (value: str)
+    * disabled (value: str)
         Emitted after plugin named `value` has been added to the block list.
     """
 
@@ -73,19 +79,20 @@ class NapariPluginManager(PluginManager):
         with self.discovery_blocked():
             self.add_hookspecs(hook_specifications)
 
-        self._sample_data: Dict[str, Dict[str, SampleDict]] = dict()
+        # dicts to store maps from extension -> plugin_name
+        self._extension2reader: Dict[str, str] = {}
+        self._extension2writer: Dict[str, str] = {}
+
+        self._sample_data: Dict[str, Dict[str, SampleDict]] = {}
         self._dock_widgets: Dict[
             str, Dict[str, Tuple[WidgetCallable, Dict[str, Any]]]
-        ] = dict()
-        self._function_widgets: Dict[
-            str, Dict[str, Callable[..., Any]]
-        ] = dict()
+        ] = {}
+        self._function_widgets: Dict[str, Dict[str, Callable[..., Any]]] = {}
+        self._theme_data: Dict[str, Dict[str, Theme]] = dict()
 
         if sys.platform.startswith('linux') and running_as_bundled_app():
             sys.path.append(user_site_packages())
 
-        _magicgui.register_types_with_magicgui()
-
     def _initialize(self):
         with self.discovery_blocked():
             self.register(_builtins, name='builtins')
@@ -94,6 +101,13 @@ class NapariPluginManager(PluginManager):
 
                 self.register(_skimage_data, name='scikit-image')
 
+            from ..settings import get_settings
+
+            # dicts to store maps from extension -> plugin_name
+            plugin_settings = get_settings().plugins
+            self._extension2reader.update(plugin_settings.extension2reader)
+            self._extension2writer.update(plugin_settings.extension2writer)
+
     def register(
         self, namespace: Any, name: Optional[str] = None
     ) -> Optional[str]:
@@ -114,13 +128,18 @@ class NapariPluginManager(PluginManager):
 
         plugin = super().unregister(name_or_object)
 
-        # remove widgets, sample data
+        # unregister any theme that was associated with the
+        # unregistered plugin
+        self.unregister_theme_colors(_name)
+
+        # remove widgets, sample data, theme data
         for _dict in (
             self._dock_widgets,
             self._sample_data,
+            self._theme_data,
             self._function_widgets,
         ):
-            _dict.pop(_name, None)
+            _dict.pop(_name, None)  # type: ignore
 
         self.events.unregistered(value=_name)
 
@@ -140,7 +159,7 @@ class NapariPluginManager(PluginManager):
             # let's reregister.  # TODO: might be able to be more direct here.
             self.discover()
 
-        SETTINGS.plugins.disabled_plugins = set(self._blocked)
+        get_settings().plugins.disabled_plugins = set(self._blocked)
 
     def call_order(self, first_result_only=True) -> CallOrderDict:
         """Returns the call order from the plugin manager.
@@ -153,7 +172,7 @@ class NapariPluginManager(PluginManager):
             sooner.
         """
 
-        order = {}
+        order: CallOrderDict = {}
         for spec_name, caller in self.hooks.items():
             # no need to save call order unless we only use first result
             if first_result_only and not caller.is_firstresult:
@@ -168,10 +187,10 @@ class NapariPluginManager(PluginManager):
         return order
 
     def set_call_order(self, new_order: CallOrderDict):
-        """Sets the plugin manager call order to match SETTINGS plugin values.
+        """Sets the plugin manager call order to match settings plugin values.
 
         Note: Run this after load_settings_plugin_defaults, which
-        sets the default values in SETTINGS.
+        sets the default values in settings.
 
         Parameters
         ----------
@@ -229,10 +248,11 @@ class NapariPluginManager(PluginManager):
             warn(message=warn_message)
             return
 
-        _data = {}
-        for name, datum in list(data.items()):
-            if isinstance(datum, dict):
-                if 'data' not in datum or 'display_name' not in datum:
+        _data: Dict[str, SampleDict] = {}
+        for name, _datum in list(data.items()):
+            if isinstance(_datum, dict):
+                datum: SampleDict = _datum
+                if 'data' not in _datum or 'display_name' not in _datum:
                     warn_message = trans._(
                         'In {hook_name!r}, plugin {plugin_name!r} provided an invalid dict object for key {name!r} that does not have required keys: "data" and "display_name". Ignoring',
                         deferred=True,
@@ -243,7 +263,7 @@ class NapariPluginManager(PluginManager):
                     warn(message=warn_message)
                     continue
             else:
-                datum = {'data': datum, 'display_name': name}
+                datum = {'data': _datum, 'display_name': name}
 
             if not (
                 callable(datum['data'])
@@ -292,14 +312,97 @@ class NapariPluginManager(PluginManager):
             (p, s) for p in self._sample_data for s in self._sample_data[p]
         )
 
+    # THEME DATA ------------------------------------
+
+    def register_theme_colors(
+        self,
+        data: Dict[str, Dict[str, Union[str, Tuple, List]]],
+        hookimpl: HookImplementation,
+    ):
+        """Register theme data dict returned by `napari_experimental_provide_theme`.
+
+        The `theme` data should be provided as an iterable containing dictionary
+        of values, where the ``folder`` value will be used as theme name.
+        """
+        plugin_name = hookimpl.plugin_name
+        hook_name = '`napari_experimental_provide_theme`'
+        if not isinstance(data, Dict):
+            warn_message = trans._(
+                'Plugin {plugin_name!r} provided a non-dict object to {hook_name!r}: data ignored',
+                deferred=True,
+                plugin_name=plugin_name,
+                hook_name=hook_name,
+            )
+            warn(message=warn_message)
+            return
+
+        _data = {}
+        for theme_name, theme_colors in data.items():
+            try:
+                theme = Theme.parse_obj(theme_colors)
+                register_theme(theme_name, theme)
+                _data[theme_name] = theme
+            except (KeyError, ValidationError) as err:
+                warn_msg = trans._(
+                    "In {hook_name!r}, plugin {plugin_name!r} provided an invalid dict object for creating themes. {err!r}",
+                    deferred=True,
+                    hook_name=hook_name,
+                    plugin_name=plugin_name,
+                    err=err,
+                )
+                warn(message=warn_msg)
+                continue
+
+        if plugin_name not in self._theme_data:
+            self._theme_data[plugin_name] = {}
+        self._theme_data[plugin_name].update(_data)
+
+    def unregister_theme_colors(self, plugin_name: str):
+        """Unregister theme data from napari."""
+        if plugin_name not in self._theme_data:
+            return
+
+        # unregister all themes that were provided by the plugins
+        for theme_name in self._theme_data[plugin_name]:
+            unregister_theme(theme_name)
+
+        # since its possible that disabled/removed plugin was providing the
+        # current theme, we check for this explicitly and if this the case,
+        # theme is automatically changed to default `dark` theme
+        settings = get_settings()
+        current_theme = settings.appearance.theme
+        if current_theme in self._theme_data[plugin_name]:
+            settings.appearance.theme = "dark"  # type: ignore
+            warnings.warn(
+                message=trans._(
+                    "The current theme {current_theme!r} was provided by the plugin {plugin_name!r} which was disabled or removed. Switched theme to the default.",
+                    deferred=True,
+                    plugin_name=plugin_name,
+                    current_theme=current_theme,
+                )
+            )
+
+    def discover_themes(self):
+        """Trigger discovery of theme plugins.
+
+        As a "historic" hook, this should only need to be called once.
+        (historic here means that even plugins that are discovered after this
+        is called will be added.)
+        """
+        if self._theme_data:
+            return
+        self.hook.napari_experimental_provide_theme.call_historic(
+            result_callback=partial(self.register_theme_colors), with_impl=True
+        )
+
     # FUNCTION & DOCK WIDGETS -----------------------
 
-    def iter_widgets(self) -> Iterator[Tuple[str, Tuple[str, Dict]]]:
+    def iter_widgets(self) -> Iterator[Tuple[str, Tuple[str, Dict[str, Any]]]]:
         from itertools import chain, repeat
 
         dock_widgets = zip(repeat("dock"), self._dock_widgets.items())
         func_widgets = zip(repeat("func"), self._function_widgets.items())
-        yield from chain(dock_widgets, func_widgets)
+        yield from chain(dock_widgets, func_widgets)  # type: ignore [misc]
 
     def register_dock_widget(
         self,
@@ -497,3 +600,123 @@ class NapariPluginManager(PluginManager):
                 raise KeyError(msg)
 
         return plg_wdgs[widget_name]
+
+    def get_reader_for_extension(self, extension: str) -> Optional[str]:
+        """Return reader plugin assigned to `extension`, or None."""
+        return self._get_plugin_for_extension(extension, type_='reader')
+
+    def assign_reader_to_extensions(
+        self, reader: str, extensions: Union[str, Iterable[str]]
+    ) -> None:
+        """Assign a specific reader plugin to `extensions`.
+
+        Parameters
+        ----------
+        reader : str
+            Name of a plugin offering a reader hook.
+        extensions : Union[str, Iterable[str]]
+            Name(s) of extensions to always write with `reader`
+        """
+        from ..settings import get_settings
+
+        self._assign_plugin_to_extensions(reader, extensions, type_='reader')
+        extension2readers = get_settings().plugins.extension2reader
+        get_settings().plugins.extension2reader = {
+            **extension2readers,
+            **self._extension2reader,
+        }
+
+    def get_writer_for_extension(self, extension: str) -> Optional[str]:
+        """Return writer plugin assigned to `extension`, or None."""
+        return self._get_plugin_for_extension(extension, type_='writer')
+
+    def assign_writer_to_extensions(
+        self, writer: str, extensions: Union[str, Iterable[str]]
+    ) -> None:
+        """Assign a specific writer plugin to `extensions`.
+
+        Parameters
+        ----------
+        writer : str
+            Name of a plugin offering a writer hook.
+        extensions : Union[str, Iterable[str]]
+            Name(s) of extensions to always write with `writer`
+        """
+        from ..settings import get_settings
+
+        self._assign_plugin_to_extensions(writer, extensions, type_='writer')
+        get_settings().plugins.extension2writer = self._extension2writer
+
+    def _get_plugin_for_extension(
+        self, extension: str, type_: str
+    ) -> Optional[str]:
+        """helper method for public get_<type_>_for_extension functions."""
+        ext_map = getattr(self, f'_extension2{type_}', None)
+        if ext_map is None:
+            raise ValueError(
+                trans._(
+                    "invalid plugin type: {type_!r}",
+                    deferred=True,
+                    type_=type_,
+                )
+            )
+
+        if not extension.startswith("."):
+            extension = f".{extension}"
+
+        plugin = ext_map.get(extension)
+        # make sure it's still an active plugin
+        if plugin and (plugin not in self.plugins):
+            del self.ext_map[plugin]
+            return None
+        return plugin
+
+    def _assign_plugin_to_extensions(
+        self,
+        plugin: str,
+        extensions: Union[str, Iterable[str]],
+        type_: Optional[str] = None,
+    ) -> None:
+        """helper method for public assign_<type_>_to_extensions functions."""
+        caller: HookCaller = getattr(self.hook, f'napari_get_{type_}', None)
+        if caller is None:
+            raise ValueError(
+                trans._(
+                    "invalid plugin type: {type_!r}",
+                    deferred=True,
+                    type_=type_,
+                )
+            )
+
+        plugins = caller.get_hookimpls()
+        if plugin not in {p.plugin_name for p in plugins}:
+            msg = trans._(
+                "{plugin!r} is not a valid {type_} plugin name",
+                plugin=plugin,
+                type_=type_,
+                deferred=True,
+            )
+            raise ValueError(msg)
+
+        ext_map = getattr(self, f'_extension2{type_}')
+        if isinstance(extensions, str):
+            extensions = [extensions]
+        for ext in extensions:
+            if not ext.startswith("."):
+                ext = f".{ext}"
+            ext_map[ext] = plugin
+
+            # give warning that plugin *may* not be able to read that extension
+            try:
+                func = caller._call_plugin(plugin, path=f'_testing_{ext}')
+            except Exception:
+                pass
+            if func is None:
+                msg = trans._(
+                    'plugin {plugin!r} did not return a {type_} function when provided a path ending in {ext!r}. This *may* indicate a typo?',
+                    deferred=True,
+                    plugin=plugin,
+                    type_=type_,
+                    ext=ext,
+                )
+                warn(msg)
