@@ -12,28 +12,23 @@ of those custom classes, magicgui will know what to do with it.
 from __future__ import annotations
 
 import weakref
-from functools import lru_cache
-from typing import TYPE_CHECKING, Any, List, Optional, Tuple, Type
+from functools import lru_cache, partial
+from typing import TYPE_CHECKING, Any, List, Optional, Set, Tuple, Type
 
-import toolz as tz
+from typing_extensions import get_args
+
+from ..utils._proxies import PublicOnlyProxy
 
 if TYPE_CHECKING:
+    from concurrent.futures import Future
+
     from magicgui.widgets._bases import CategoricalWidget
 
+    from .._qt.qthreading import FunctionWorker
     from ..layers import Layer
     from ..viewer import Viewer
 
 
-# TODO: make register_type a better decorator upstream
-@tz.curry
-def register_type(type_: type, **kwargs) -> type:
-    """Decorator to register a type with magicgui."""
-    import magicgui
-
-    magicgui.register_type(type_, **kwargs)
-    return type_
-
-
 def add_layer_data_to_viewer(gui, result, return_type):
     """Show a magicgui result in the viewer.
 
@@ -155,6 +150,113 @@ def add_layer_data_tuples_to_viewer(gui, result, return_type):
             viewer._add_layer_from_data(*layer_datum)
 
 
+_FUTURES: Set[Future] = set()
+
+
+def add_worker_data(
+    widget, worker: FunctionWorker, return_type, _from_tuple=True
+):
+    """Handle a thread_worker object returned from a magicgui widget.
+
+    This allows someone annotate their magicgui with a return type of
+    `FunctionWorker[...]`, create a napari thread worker (e.g. with the
+    ``@thread_worker`` decorator), then simply return the worker.  We will hook up
+    the `returned` signal to the machinery to add the result of the
+    long-running function to the viewer.
+
+    Parameters
+    ----------
+    widget : MagicGui
+        The instantiated MagicGui widget.  May or may not be docked in a
+        dock widget.
+    worker : WorkerBase
+        An instance of `napari._qt.qthreading.WorkerBase`, on worker.returned,
+        the result will be added to the viewer.
+    return_type : type
+        The return annotation that was used in the decorated function.
+    _from_tuple : bool, optional
+        (only for internal use). True if the worker returns `LayerDataTuple`,
+        False if it returns one of the `LayerData` types.
+
+    Examples
+    --------
+
+    .. code-block:: python
+
+        @magicgui
+        def my_widget(...) -> FunctionWorker[ImageData]:
+
+            @thread_worker
+            def do_something_slowly(...) -> ImageData:
+                ...
+
+            return do_something_slowly(...)
+
+    """
+
+    cb = (
+        add_layer_data_tuples_to_viewer
+        if _from_tuple
+        else add_layer_data_to_viewer
+    )
+    _return_type = get_args(return_type)[0]
+    worker.signals.returned.connect(
+        partial(cb, widget, return_type=_return_type)
+    )
+
+
+def add_future_data(gui, future, return_type, _from_tuple=True):
+    """Process a Future object from a magicgui widget.
+
+    This function will be called when a magicgui-decorated function has a
+    return annotation of one of the `napari.types.<layer_name>Data` ... and
+    will add the data in ``result`` to the current viewer as the corresponding
+    layer type.
+
+    Parameters
+    ----------
+    gui : FunctionGui
+        The instantiated magicgui widget.  May or may not be docked in a
+        dock widget.
+    future : Future
+        An instance of `concurrent.futures.Future` (or any third-party) object
+        with the same interface, that provides `add_done_callback` and `result`
+        methods.  When the future is `done()`, the `result()` will be added
+        to the viewer.
+    return_type : type
+        The return annotation that was used in the decorated function.
+    _from_tuple : bool, optional
+        (only for internal use). True if the future returns `LayerDataTuple`,
+        False if it returns one of the `LayerData` types.
+    """
+    from .._qt.utils import Sentry
+
+    # get the actual return type from the Future type annotation
+    _return_type = get_args(return_type)[0]
+
+    if _from_tuple:
+        # when the future is done, add layer data to viewer, dispatching
+        # to the appropriate method based on the Future data type.
+
+        def _on_future_ready():
+            add_layer_data_tuples_to_viewer(gui, future.result(), return_type)
+            _FUTURES.discard(future)
+
+    else:
+
+        def _on_future_ready():
+            add_layer_data_to_viewer(gui, future.result(), _return_type)
+            _FUTURES.discard(future)
+
+    # some future types (such as a dask Future) will call the callback in
+    # another thread, which wont always work here.  So we create a very small
+    # QObject that can signal back to the main thread to call `_on_done`.
+    sentry = Sentry()
+    sentry.alerted.connect(_on_future_ready)
+    future.add_done_callback(sentry.alert)
+    _FUTURES.add(future)
+
+
 def find_viewer_ancestor(widget) -> Optional[Viewer]:
     """Return the Viewer object if it is an ancestor of ``widget``, else None.
 
@@ -168,6 +270,8 @@ def find_viewer_ancestor(widget) -> Optional[Viewer]:
     viewer : napari.Viewer or None
         Viewer instance if one exists, else None.
     """
+    from .._qt.widgets.qt_viewer_dock_widget import QtViewerDockWidget
+
     # magicgui v0.2.0 widgets are no longer QWidget subclasses, but the native
     # widget is available at widget.native
     if hasattr(widget, 'native') and hasattr(widget.native, 'parent'):
@@ -175,12 +279,24 @@ def find_viewer_ancestor(widget) -> Optional[Viewer]:
     else:
         parent = widget.parent()
     while parent:
-        if hasattr(parent, 'qt_viewer'):
-            return parent.qt_viewer.viewer
+        if hasattr(parent, '_qt_viewer'):  # QMainWindow
+            return parent._qt_viewer.viewer
+        if isinstance(parent, QtViewerDockWidget):  # DockWidget
+            qt_viewer = parent._ref_qt_viewer()
+            if qt_viewer is not None:
+                return qt_viewer.viewer
+            return None
         parent = parent.parent()
     return None
 
 
+def proxy_viewer_ancestor(widget) -> Optional[PublicOnlyProxy[Viewer]]:
+    viewer = find_viewer_ancestor(widget)
+    if viewer:
+        return PublicOnlyProxy(viewer)
+    return None
+
+
 def get_layers(gui: CategoricalWidget) -> List[Layer]:
     """Retrieve layers matching gui.annotation, from the Viewer the gui is in.
 
@@ -263,7 +379,7 @@ def _make_choice_data_setter(gui: CategoricalWidget, choice_name: str):
     Note, using lru_cache here so that the **same** function object is returned
     if you call this twice for the same widget/choice_name combination. This is
     so that when we connect it above in `layer.events.data.connect()`, it will
-    only get connected once (because .connect() will not add a specific callback
+    only get connected once (because ``.connect()`` will not add a specific callback
     more than once)
     """
     gui_ref = weakref.ref(gui)
@@ -296,6 +412,7 @@ def add_layer_to_viewer(gui, result: Any, return_type: Type[Layer]) -> None:
     >>> @magicgui
     ... def make_layer() -> napari.layers.Image:
     ...     return napari.layers.Image(np.random.rand(64, 64))
+
     """
     if result is None:
         return
