@@ -1,4 +1,6 @@
-from typing import TYPE_CHECKING
+import typing
+from typing import TYPE_CHECKING, Optional
+from weakref import WeakSet
 
 import magicgui as mgui
 
@@ -10,7 +12,7 @@ if TYPE_CHECKING:
     from ._qt.qt_main_window import Window
 
 
-@mgui.register_type(bind=_magicgui.find_viewer_ancestor)
+@mgui.register_type(bind=_magicgui.proxy_viewer_ancestor)
 class Viewer(ViewerModel):
     """Napari ndarray viewer.
 
@@ -31,6 +33,7 @@ class Viewer(ViewerModel):
     """
 
     _window: 'Window' = None  # type: ignore
+    _instances: typing.ClassVar[WeakSet] = WeakSet()
 
     def __init__(
         self,
@@ -52,6 +55,7 @@ class Viewer(ViewerModel):
         from .window import Window
 
         self._window = Window(self, show=show)
+        self._instances.add(self)
 
     # Expose private window publically. This is needed to keep window off pydantic model
     @property
@@ -71,10 +75,10 @@ class Viewer(ViewerModel):
             give (list/tuple/str) then the variable values looked up in the
             callers frame.
         """
-        if self.window.qt_viewer._console is None:
+        if self.window._qt_viewer._console is None:
             return
         else:
-            self.window.qt_viewer.console.push(variables)
+            self.window._qt_viewer.console.push(variables)
 
     def screenshot(self, path=None, *, canvas_only=True, flash: bool = True):
         """Take currently displayed screen and convert to an image array.
@@ -98,11 +102,9 @@ class Viewer(ViewerModel):
             Numpy array of type ubyte and shape (h, w, 4). Index [0, 0] is the
             upper-left corner of the rendered region.
         """
-        if canvas_only:
-            image = self.window.qt_viewer.screenshot(path=path, flash=flash)
-        else:
-            image = self.window.screenshot(path=path, flash=flash)
-        return image
+        return self.window.screenshot(
+            path=path, flash=flash, canvas_only=canvas_only
+        )
 
     def show(self, *, block=False):
         """Resize, show, and raise the viewer window."""
@@ -124,9 +126,33 @@ class Viewer(ViewerModel):
             # https://github.com/napari/napari/issues/1500
             for layer in self.layers:
                 chunk_loader.on_layer_deleted(layer)
+        self._instances.discard(self)
+
+    @classmethod
+    def close_all(cls) -> int:
+        """
+        Class metod, Close all existing viewer instances.
+
+        This is mostly exposed to avoid leaking of viewers when running tests.
+        As having many non-closed viewer can adversely affect performances.
+
+        It will return the number of viewer closed.
+
+        Returns
+        -------
+        int :
+            number of viewer closed.
+
+        """
+        # copy to not iterate while changing.
+        viewers = [v for v in cls._instances]
+        ret = len(viewers)
+        for viewer in viewers:
+            viewer.close()
+        return ret
 
 
-def current_viewer() -> Viewer:
+def current_viewer() -> Optional[Viewer]:
     """Return the currently active napari viewer."""
     try:
         from napari._qt.qt_main_window import _QtMainWindow
